package org.jo.jmx;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;

import org.jo.dto.ObservationBean;
import org.springframework.context.Lifecycle;

/**
 * Interactive {@link ObservationBean} entry object.
 */
public class ObservationEntry implements Lifecycle {
	private ObservationBean observation;
	private BufferedReader inputReader;
	private InputStream inputStream = System.in;
	private PrintStream outputStream = System.out;
	private PrintStream errorStream = System.err;
	private boolean running = false;

	public ObservationEntry(ObservationBean observation) {
		this.observation = observation;
	}

//	public void setInputStream(InputStream inputStream) {
//		this.inputStream = inputStream;
//	}
//
//	public void setOutputStream(PrintStream outputStream) {
//		this.outputStream = outputStream;
//	}
//
//	public void setErrorStream(PrintStream errorStream) {
//		this.errorStream = errorStream;
//	}

	// ---------------------------------------
	// -- Spring Lifecycle interface methods
	// ---------------------------------------
	@Override
	public void start() {
		this.running = true;

		if (this.inputReader == null)
			this.inputReader = new BufferedReader(new InputStreamReader(
					inputStream));

		try {
			observationInput();
		} catch (IOException ex) {
			throw new RuntimeException(ex);
		}
	}

	@Override
	public boolean isRunning() {
		return running;
	}

	@Override
	public void stop() {
		outputStream
				.print("\n\nReceived shutdown request, press any key to exit.\n");
		this.running = false;
	}

	/**
	 * Start accepting user input for creating new {@link ObservationBean}
	 * transactions. Loops until user specifies 'n' when prompted about entering
	 * another events.
	 */
	public void observationInput() throws IOException {

		String enterObservation;

		outputStream.printf("Welcome to the observation system entry UI\n\n");
		outputStream.printf("Please enter the following information \n");

		do {
			try {
				outputStream.printf("\n");
				String strNbrEvent = prompt("The number of observed events");
				observation.setNbrEvent(Integer.valueOf(strNbrEvent));
				outputStream.printf("Result: %s\n\n", observation);

			} catch (Exception ex) {
				errorStream.printf(
						"\nobservation entry error! message was: %s\n\n", ex
								.getMessage());
			}

			do {
				enterObservation = prompt("Would you like to enter another observed events? [y/n]");
			} while (isRunning() && !enterObservation.equalsIgnoreCase("y")
					&& !enterObservation.equalsIgnoreCase("n"));

		} while (isRunning() && enterObservation.equalsIgnoreCase("y"));

		outputStream.printf("\nThank you, goodbye.\n");
	}

	/**
	 * Provide a user prompt; accept and return user input.
	 */
	private String prompt(String desiredInput) throws IOException {
		outputStream.print(desiredInput + ": ");
		return inputReader.readLine();
	}
}
